"""
todo
better hit boxes
goals
better graphics
music
cutscenes
"""

import math
import random
import pygame
from pygame.locals import *
import pygame.gfxdraw
import pygame.image

worldWidth = 640
worldHeight = 480
numberOfCreatures = 20
enemySprite2 = None
playerSprite = None
heartIcon = None
pillIcon = None
hitSound = None
gameSpeed = 1.0
screenFadeout = 0
creatureList = []
itemList = []
player = None
currentRound = 0

def rot_center(image, angle):
    """rotate an image while keeping its center and size"""
    orig_rect = image.get_rect()
    rot_image = pygame.transform.rotate(image, angle)
    rot_rect = orig_rect.copy()
    rot_rect.center = rot_image.get_rect().center
    rot_image = rot_image.subsurface(rot_rect).copy()
    return rot_image

class Animation:
	def __init__(self):
		self.images = []
		self.frameLengths = []
	def setNextFrame(self, image, frameLength):
		imageSurface = pygame.image.load(image)
		imageSurface = pygame.transform.scale2x(imageSurface)
		self.images.append([])
		self.images[-1].append(imageSurface)
		for i in range (1, 8):
			tempSurface = rot_center(imageSurface, i/8.00 * 360)
			self.images[-1].append(tempSurface)			
		self.frameLengths.append(frameLength)

class AnimationInstance:
	def __init__(self, animation):
		self.frame = 0
		self.animation = animation
		self.paused = 0
	def blit(self, x, y, rotation):
		frameIndex = int(rotation/2.0 * 8.0 + .5)
		if frameIndex > 7:
			frameIndex = 0
		s.blit(self.animation.images[self.frame][frameIndex], (x, y))
		if self.paused == 0:
			self.frame += 1
			if self.frame == len(self.animation.images):
				self.frame = 0

class Object:
	def __init__(self):
		self.collisionX = 0
		self.collisionY = 0
		self.collisionBoxWidth = 64
		self.collisionBoxHeight = 64
		self.collisionBox.x = self.x
		self.collisionBox.y = self.y
	def isWithinBounds(self, target):
		if target.x >= self.x + self.collisionX + self.collisionBoxWidth:
			return 0
		if target.x + target.collisionBoxWidth < self.x + self.collisionX:
			return 0
		if target.y >= self.y + self.collisionY + self.collisionBoxHeight:
			return 0
		if target.y + target.collisionBoxHeight < self.y + self.collisionY:
			return 0
		if math.hypot((self.x + 32) - (target.x + 32), (self.y + 32) - (target.y + 32)) < 36:
			return 1
		return 0
	def draw(self):
		self.animation.blit(self.x, self.y, self.rotation)

class Creature(Object):
	def __init__(self):
		self.x = random.randrange(worldWidth)
		self.y = random.randrange(worldHeight)
		flag = 0
		while 1:
			if flag == 1:
				break
			flag = 1
			for i in creatureList:
				if math.hypot((self.x + 32) - (i.x + 32), (self.y + 32) - (i.y + 32)) < 36:
					self.x = random.randrange(worldWidth)
					self.y = random.randrange(worldHeight)
					flag = 0
					break
		self.speed = 5 * gameSpeed
		self.rotation = random.random() * 2
		self.frame = 0
		self.collisionX = 0
		self.collisionY = 0
		self.collisionBoxWidth = 64
		self.collisionBoxHeight = 64
		self.justRotated = 0
		self.animation = AnimationInstance(enemySprite2)
	def checkForCollisions(self):
		for i in creatureList:
			if i != self:
				if i.isWithinBounds(self):
					return 1
		return 0
	def update(self):
		oldX = self.x
		oldY = self.y
		oldRotation = self.rotation
		self.x = self.x + math.sin(self.rotation * math.pi) * self.speed
		self.y = self.y + math.cos(self.rotation * math.pi) * self.speed
		if self.x >= worldWidth - 64:
			self.x = worldWidth - 64
			self.rotation = 2 - self.rotation
		if self.y >= worldHeight - 64:
			self.y = worldHeight - 64
			self.rotation = 1 - self.rotation
			if self.rotation < 0:
				self.rotation = 2 + self.rotation
		if self.x < 0:
			self.x = 0
			self.rotation = 2 - self.rotation
		if self.y < 0:
			self.y = 0
			self.rotation = 1 - self.rotation
			if self.rotation < 0:
				self.rotation = 2 + self.rotation
		if self.checkForCollisions():
			# there's a collision, so move everything back
			self.x = oldX
			self.y = oldY
			if self.justRotated == 0:
				self.rotation += 1
				if self.rotation > 2:
					self.rotation -= 2
				self.justRotated = 5
		if self.justRotated > 0:
			self.justRotated -= 1
		
class Player(Object):
	def __init__(self):
		self.x = worldWidth / 2
		self.y = worldHeight / 2
		self.speed = 9 * 1.5
		self.rotation = 1
		self.collisionX = 0
		self.collisionY = 0
		self.collisionBoxWidth = 64
		self.collisionBoxHeight = 64
		self.health = 3
		self.goalsAchieved = 0
		self.justHit = 20
		self.dying = 0
		self.roundEnding = 0
		self.animation = AnimationInstance(playerSprite)
	def goalAchieved(self):
		self.goalsAchieved += 1
		if self.goalsAchieved == 3:
			self.roundEnding = 1
			global currentRound
			currentRound += 1
	def draw(self):
		if self.justHit == 0 or self.justHit % 2 == 0:
			self.animation.blit(self.x, self.y, self.rotation)
		# draw health
		for i in range (0, self.health):
			s.blit(heartIcon, (16 + i * 40,16))
	def update(self):
		global screenFadeout
		global currentRound
		for event in pygame.event.get():
			if event.type == pygame.QUIT:
				pygame.quit()
			if event.type == pygame.KEYDOWN:
				if event.key == pygame.K_ESCAPE:
					pygame.quit()
					exit(0)
				if currentRound == 0:
					currentRound = 1
					pygame.mixer.music.play(-1)
				if self.dying > 30:
					gameInstance.reset()
					self.dying = 0
					screenFadeout = 0
				elif self.roundEnding > 30:
					if currentRound > 3:
						pygame.quit()
					gameInstance.reset()
					self.roundEnding = 0
					screenFadeout = 0
					global gameSpeed
					gameSpeed += .5
				"""
				if event.unicode == 's':
					# save
					file = open('save', 'w')
					pickle.dump(creatureList,file)
					file.close()
				if event.unicode == 'l':
					# load
					unpicklefile = open('save', 'r')
					creatureList = pickle.load(unpicklefile)
					unpicklefile.close()
				"""						
		if currentRound > 0:
			if self.dying == 0 and self.roundEnding == 0:
				oldX = self.x
				oldY = self.y
				key=pygame.key.get_pressed()
				if key[pygame.K_DOWN] and key[pygame.K_RIGHT]:
					self.rotation = .25
				elif key[pygame.K_DOWN] and key[pygame.K_LEFT]:
					self.rotation = 1.75
				elif key[pygame.K_UP] and key[pygame.K_RIGHT]:
					self.rotation = .75
				elif key[pygame.K_UP] and key[pygame.K_LEFT]:
					self.rotation = 1.25
				elif key[pygame.K_DOWN]:
					self.rotation = 0
				elif key[pygame.K_UP]:
					self.rotation = 1
				elif key[pygame.K_RIGHT]:
					self.rotation = .5
				elif key[pygame.K_LEFT]:
					self.rotation = 1.5
				self.x = self.x + math.sin(self.rotation * math.pi) * self.speed
				self.y = self.y + math.cos(self.rotation * math.pi) * self.speed
				if self.x >= worldWidth - 64:
					self.x = worldWidth - 64
				if self.y >= worldHeight - 64:
					self.y = worldHeight - 64
				if self.x < 0:
					self.x = 0
				if self.y < 0:
					self.y = 0
				for i in creatureList:
					if self.justHit == 0:
						if i.isWithinBounds(self):
							self.x = oldX
							self.y = oldY
							hitSound.play()
							self.health -= 1
							if self.health == 0:
								self.dying = 1
								self.animation.paused = 1
							else:
								self.justHit = 20
				if self.justHit > 0:
					self.justHit -= 1
				for i in itemList:
					if i.isWithinBounds(self):
						i.touched()
						itemList.remove(i)
			elif self.dying != 0:
				self.dying += 1
				screenFadeout += 10
			elif self.roundEnding != 0:
				self.roundEnding += 1
				screenFadeout += 10

class Health():
	def __init__(self):
		self.x = random.randrange(worldWidth - 32)
		self.y = random.randrange(worldHeight - 32)
		self.collisionX = 0
		self.collisionY = 0
		self.collisionBoxWidth = 64
		self.collisionBoxHeight = 64
	def draw(self):
		s.blit(heartIcon, (self.x, self.y))
	def update(self):
		self.x += 1
	def touched(self):
		player.health += 1
	def isWithinBounds(self, target):
		if target.x >= self.x + self.collisionX + self.collisionBoxWidth:
			return 0
		if target.x + target.collisionBoxWidth < self.x + self.collisionX:
			return 0
		if target.y >= self.y + self.collisionY + self.collisionBoxHeight:
			return 0
		if target.y + target.collisionBoxHeight < self.y + self.collisionY:
			return 0
		if math.hypot((self.x + 32) - (target.x + 32), (self.y + 32) - (target.y + 32)) < 64:
			return 1
		return 0


class Goal():
	def __init__(self):
		self.x = random.randrange(worldWidth)
		self.y = random.randrange(worldHeight)
		self.collisionX = 0
		self.collisionY = 0
		self.collisionBoxWidth = 64
		self.collisionBoxHeight = 64
	def draw(self):
		s.blit(pillIcon, (self.x, self.y))
	def touched(self):
		player.goalAchieved()
	def isWithinBounds(self, target):
		if target.x >= self.x + self.collisionX + self.collisionBoxWidth:
			return 0
		if target.x + target.collisionBoxWidth < self.x + self.collisionX:
			return 0
		if target.y >= self.y + self.collisionY + self.collisionBoxHeight:
			return 0
		if target.y + target.collisionBoxHeight < self.y + self.collisionY:
			return 0
		if math.hypot((self.x + 16) - (target.x + 32), (self.y + 16) - (target.y + 32)) < 16 + 32:
			return 1
		return 0

class Game:
	def reset(self):
		global creatureList
		global itemList
		global player
		creatureList = []
		itemList = []
		itemList.append(Goal())
		i = 0
		while i < numberOfCreatures:
			creatureList.append(Creature())
			i = i + 1
		player = None
		player = Player()
	def mainLoop(self):
		frameCounter = pygame.time.get_ticks()
		frame = pygame.Surface((worldWidth, worldHeight), pygame.SRCALPHA, 32)
		healthTimer = 0
		pillTimer = 0
		while 1:
			t = pygame.time.get_ticks()
			if frameCounter + 70 > t:
				pygame.time.wait(frameCounter + 70 - t)
			frameCounter = pygame.time.get_ticks()

			s.fill((120, 120, 120))
			player.update()
			if currentRound > 0:
				healthTimer += 1
				pillTimer += 1
				if healthTimer > 75:
					itemList.append(Health())
					healthTimer = 0
				if pillTimer > 150:
					itemList.append(Goal())
					pillTimer = 0
				for i in creatureList:
					i.update()
					i.draw()
				for i in itemList:
					i.draw()
				player.draw()
				darken=pygame.Surface(pygame.display.get_surface().get_size())
				if screenFadeout > 0:
					healthTimer = 0
					pillTimer = 0
					screen.blit(s, (0, 0))
					darken.fill((0,0,0))
					darken.set_alpha(screenFadeout)
					screen.blit(darken, (0, 0))
					if player.dying != 0:
						font = pygame.font.Font(None, 50)
						text = font.render("You have died.", 1, (255, 255, 255))
						textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2)
						screen.blit(text, textpos)
						text = font.render("Press any key to try again.", 1, (255, 255, 255))
						textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 + 100)
						screen.blit(text, textpos)
					elif currentRound <= 3:
						font = pygame.font.Font(None, 50)
						text = font.render("You have collected enough hormones.", 1, (255, 255, 255))
						textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2)
						screen.blit(text, textpos)
						text = font.render("Press any key to proceed", 1, (255, 255, 255))
						textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 + 100)
						screen.blit(text, textpos)
						text = font.render("to the next round.", 1, (255, 255, 255))
						textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 + 150)
						screen.blit(text, textpos)
					else:
						font = pygame.font.Font(None, 50)
						text = font.render("You have collected enough hormones.", 1, (255, 255, 255))
						textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2)
						screen.blit(text, textpos)
						text = font.render("You win!", 1, (255, 255, 255))
						textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 + 100)
						screen.blit(text, textpos)
						text = font.render("You have become a", 1, (255, 255, 255))
						textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 + 150)
						screen.blit(text, textpos)
						text = font.render("bear mode skinhead.", 1, (255, 255, 255))
						textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 + 200)
						screen.blit(text, textpos)
				else:
					screen.blit(s, (0, 0))
			else:
				darken=pygame.Surface(pygame.display.get_surface().get_size())
				screen.blit(s, (0, 0))
				darken.fill((0,0,0))
				screen.blit(darken, (0, 0))
				font = pygame.font.Font(None, 20)
				text = font.render("You play this game as a small teenage girl who wishes to become a large, burly male. You must", 1, (255, 255, 255))
				textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 - 200)
				screen.blit(text, textpos)

				text = font.render("traverse through a mosh pit, picking up bottles of male hormone pills, while avoiding getting", 1, (255, 255, 255))
				textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 - 180)
				screen.blit(text, textpos)

				text = font.render("hit. Use the arrow keys to navigate. Pick up hearts along the way to increase your health.", 1, (255, 255, 255))
				textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 - 160)
				screen.blit(text, textpos)

				text = font.render("After you have picked up three pill bottles, you will go on to the next round.", 1, (255, 255, 255))
				textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 - 140)
				screen.blit(text, textpos)

				text = font.render("The game ends after three rounds.", 1, (255, 255, 255))
				textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 - 120)
				screen.blit(text, textpos)

				text = font.render("Press any key to begin.", 1, (255, 255, 255))
				textpos = text.get_rect(centerx=worldWidth/2, centery=worldHeight/2 + 100)
				screen.blit(text, textpos)
				

        		pygame.display.flip()



gameInstance = Game()

pygame.init()
screen = pygame.display.set_mode((worldWidth, worldHeight))
s = pygame.Surface(screen.get_size(), pygame.SRCALPHA, 32)
pygame.key.set_repeat(15,15)
pygame.display.set_caption("mosh pit game")
enemySprite2 = Animation()
enemySprite2.setNextFrame('sprites/enemy/tiles-1.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-2.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-3.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-4.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-5.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-6.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-7.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-8.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-9.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-10.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-11.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-12.png', 0.04)
enemySprite2.setNextFrame('sprites/enemy/tiles-13.png', 0.04)

playerSprite = Animation()
playerSprite.setNextFrame('sprites/player/tiles-1.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-2.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-3.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-4.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-5.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-6.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-7.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-8.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-9.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-10.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-11.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-12.png', 0.04)
playerSprite.setNextFrame('sprites/player/tiles-13.png', 0.04)
pygame.mixer.music.load('killerrobotpenis.it')
player = Player()
hitSound = pygame.mixer.Sound('pain.wav')
try:
	heartIcon = pygame.image.load("sprites/heart.png")
except:
	print 'An error has occurred while the game was loading the image'
	exit(0)
heartIcon.convert()
try:
	pillIcon = pygame.image.load("sprites/pills.png")
except:
	print 'An error has occurred while the game was loading the image'
	exit(0)
pillIcon.convert()

gameInstance.reset()
gameInstance.mainLoop()
	
pygame.quit()
